@@ -1,7 +1,7 @@
 /*
  * config.c - parse repmgr.conf and other configuration-related functionality
  *
- * Copyright (c) 2ndQuadrant, 2010-2019
+ * Copyright (c) 2ndQuadrant, 2010-2020
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -378,13 +378,6 @@ _parse_config(t_configuration_options *options, ItemList *error_list, ItemList *
 	 */
 	options->witness_sync_interval = DEFAULT_WITNESS_SYNC_INTERVAL;
 
-	/*-------------
-	 * BDR settings
-	 *-------------
-	 */
-	options->bdr_local_monitoring_only = false;
-	options->bdr_recovery_timeout = DEFAULT_BDR_RECOVERY_TIMEOUT;
-
 	/*-------------------------
 	 * service command settings
 	 *-------------------------
@@ -569,8 +562,6 @@ _parse_config(t_configuration_options *options, ItemList *error_list, ItemList *
 void
 parse_configuration_item(t_configuration_options *options, ItemList *error_list, ItemList *warning_list, const char *name, const char *value)
 {
-	bool		known_parameter = true;
-
 	if (strcmp(name, "node_id") == 0)
 	{
 		options->node_id = repmgr_atoi(value, name, error_list, MIN_NODE_ID);
@@ -616,10 +607,8 @@ parse_configuration_item(t_configuration_options *options, ItemList *error_list,
 	{
 		if (strcmp(value, "physical") == 0)
 			options->replication_type = REPLICATION_TYPE_PHYSICAL;
-		else if (strcmp(value, "bdr") == 0)
-			options->replication_type = REPLICATION_TYPE_BDR;
 		else
-			item_list_append(error_list, _("value for \"replication_type\" must be \"physical\" or \"bdr\""));
+			item_list_append(error_list, _("value for \"replication_type\" must be \"physical\""));
 	}
 
 	/* log settings */
@@ -778,12 +767,6 @@ parse_configuration_item(t_configuration_options *options, ItemList *error_list,
 	else if (strcmp(name, "witness_sync_interval") == 0)
 		options->witness_sync_interval = repmgr_atoi(value, name, error_list, 1);
 
-	/* BDR settings */
-	else if (strcmp(name, "bdr_local_monitoring_only") == 0)
-		options->bdr_local_monitoring_only = parse_bool(value, name, error_list);
-	else if (strcmp(name, "bdr_recovery_timeout") == 0)
-		options->bdr_recovery_timeout = repmgr_atoi(value, name, error_list, 0);
-
 	/* service settings */
 	else if (strcmp(name, "pg_ctl_options") == 0)
 		strncpy(options->pg_ctl_options, value, sizeof(options->pg_ctl_options));
@@ -841,73 +824,47 @@ parse_configuration_item(t_configuration_options *options, ItemList *error_list,
 	{
 		item_list_append(warning_list,
 						 _("parameter \"cluster\" is deprecated and will be ignored"));
-		known_parameter = false;
 	}
 	else if (strcmp(name, "node") == 0)
 	{
 		item_list_append(warning_list,
 						 _("parameter \"node\" has been renamed to \"node_id\""));
-		known_parameter = false;
 	}
 	else if (strcmp(name, "upstream_node") == 0)
 	{
 		item_list_append(warning_list,
 						 _("parameter \"upstream_node\" has been removed; use \"--upstream-node-id\" when cloning a standby"));
-		known_parameter = false;
 	}
 	else if (strcmp(name, "loglevel") == 0)
 	{
 		item_list_append(warning_list,
 						 _("parameter \"loglevel\" has been renamed to \"log_level\""));
-		known_parameter = false;
 	}
 	else if (strcmp(name, "logfacility") == 0)
 	{
 		item_list_append(warning_list,
 						 _("parameter \"logfacility\" has been renamed to \"log_facility\""));
-		known_parameter = false;
 	}
 	else if (strcmp(name, "logfile") == 0)
 	{
 		item_list_append(warning_list,
 						 _("parameter \"logfile\" has been renamed to \"log_file\""));
-		known_parameter = false;
 	}
 	else if (strcmp(name, "master_reponse_timeout") == 0)
 	{
 		item_list_append(warning_list,
 						 _("parameter \"master_reponse_timeout\" has been removed; use \"async_query_timeout\" instead"));
-		known_parameter = false;
 	}
 	else if (strcmp(name, "retry_promote_interval_secs") == 0)
 	{
 		item_list_append(warning_list,
 						 _("parameter \"retry_promote_interval_secs\" has been removed; use \"primary_notification_timeout\" instead"));
-		known_parameter = false;
 	}
 	else
 	{
-		known_parameter = false;
 		log_warning(_("%s/%s: unknown name/value pair provided; ignoring"), name, value);
 	}
 
-	/*
-	 * Raise an error if a known parameter is provided with an empty
-	 * value. Currently there's no reason why empty parameters are needed;
-	 * if we want to accept those, we'd need to add stricter default
-	 * checking, as currently e.g. an empty `node_id` value will be converted
-	 * to '0'.
-	 */
-	if (known_parameter == true && !strlen(value))
-	{
-		char		error_message_buf[MAXLEN] = "";
-
-		maxlen_snprintf(error_message_buf,
-						_("\"%s\": no value provided"),
-						name);
-
-		item_list_append(error_list, error_message_buf);
-	}
 }
 
 
@@ -1112,8 +1069,6 @@ parse_time_unit_parameter(const char *name, const char *value, char *dest, ItemL
  * with these):
  *
  * - async_query_timeout
- * - bdr_local_monitoring_only
- * - bdr_recovery_timeout
  * - child_nodes_check_interval
  * - child_nodes_connected_min_count
  * - child_nodes_connected_include_witness
@@ -1180,7 +1135,8 @@ reload_config(t_configuration_options *orig_options, t_server_type server_type)
 	_parse_config(&new_options, &config_errors, &config_warnings);
 
 
-	if (server_type == PRIMARY || server_type == STANDBY)
+	if (new_options.failover == FAILOVER_AUTOMATIC
+		&& (server_type == PRIMARY || server_type == STANDBY))
 	{
 		if (new_options.promote_command[0] == '\0')
 		{
@@ -1249,24 +1205,6 @@ reload_config(t_configuration_options *orig_options, t_server_type server_type)
 		config_changed = true;
 	}
 
-	/* bdr_local_monitoring_only */
-	if (orig_options->bdr_local_monitoring_only != new_options.bdr_local_monitoring_only)
-	{
-		orig_options->bdr_local_monitoring_only = new_options.bdr_local_monitoring_only;
-		log_info(_("\"bdr_local_monitoring_only\" is now \"%s\""), new_options.bdr_local_monitoring_only == true ? "TRUE" : "FALSE");
-
-		config_changed = true;
-	}
-
-	/* bdr_recovery_timeout */
-	if (orig_options->bdr_recovery_timeout != new_options.bdr_recovery_timeout)
-	{
-		orig_options->bdr_recovery_timeout = new_options.bdr_recovery_timeout;
-		log_info(_("\"bdr_recovery_timeout\" is now \"%i\""), new_options.bdr_recovery_timeout);
-
-		config_changed = true;
-	}
-
 	/* child_nodes_check_interval */
 	if (orig_options->child_nodes_check_interval != new_options.child_nodes_check_interval)
 	{
@@ -1882,6 +1820,9 @@ modify_auto_conf(const char *data_dir, KeyValueList *items)
 	PQExpBufferData auto_conf_contents;
 
 	FILE	   *fp;
+	mode_t		um;
+	struct stat auto_conf_st;
+
 	KeyValueList config = {NULL, NULL};
 	KeyValueListCell *cell = NULL;
 
@@ -1937,7 +1878,16 @@ modify_auto_conf(const char *data_dir, KeyValueList *items)
 						  cell->key, cell->value);
 	}
 
+	stat(auto_conf.data, &auto_conf_st);
+
+	/*
+	 * Set umask so the temporary file is created in the same mode as the original
+	 * postgresql.auto.conf file.
+	 */
+	um = umask(~(auto_conf_st.st_mode));
 	fp = fopen(auto_conf_tmp.data, "w");
+	umask(um);
+
 	if (fp == NULL)
 	{
 		fprintf(stderr, "unable to open \"%s\": %s\n",
