@@ -1,7 +1,7 @@
 /*
  * dbutils.h
  *
- * Copyright (c) 2ndQuadrant, 2010-2019
+ * Copyright (c) 2ndQuadrant, 2010-2020
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,11 +60,6 @@
 	"NULL AS attached "
 
 
-
-#define BDR2_NODES_COLUMNS "node_sysid, node_timeline, node_dboid, node_name, node_local_dsn, ''"
-#define BDR3_NODES_COLUMNS "ns.node_id, 0, 0, ns.node_name, ns.interface_connstr, ns.peer_state_name"
-
-
 #define ERRBUFF_SIZE 512
 
 typedef enum
@@ -72,8 +67,7 @@ typedef enum
 	UNKNOWN = 0,
 	PRIMARY,
 	STANDBY,
-	WITNESS,
-	BDR
+	WITNESS
 } t_server_type;
 
 typedef enum
@@ -133,6 +127,8 @@ typedef enum
 typedef enum
 {
 	SLOT_UNKNOWN = -1,
+	SLOT_NOT_FOUND,
+	SLOT_NOT_PHYSICAL,
 	SLOT_INACTIVE,
 	SLOT_ACTIVE
 } ReplSlotStatus;
@@ -170,6 +166,7 @@ typedef struct
 	char		current_timestamp[MAXLEN];
 	bool		in_recovery;
 	TimeLineID	timeline_id;
+	char		timeline_id_str[MAXLEN];
 	XLogRecPtr	last_wal_receive_lsn;
 	XLogRecPtr	last_wal_replay_lsn;
 	char		last_xact_replay_timestamp[MAXLEN];
@@ -324,45 +321,6 @@ typedef struct s_connection_user
 #define T_CONNECTION_USER_INITIALIZER { "", false }
 
 
-/* represents an entry in bdr.bdr_nodes */
-typedef struct s_bdr_node_info
-{
-	char		node_sysid[MAXLEN];
-	uint32		node_timeline;
-	uint32		node_dboid;
-	char		node_name[MAXLEN];
-	char		node_local_dsn[MAXLEN];
-	char		peer_state_name[MAXLEN];
-} t_bdr_node_info;
-
-#define T_BDR_NODE_INFO_INITIALIZER { \
-	"", InvalidOid, InvalidOid, \
-    "", "", "" \
-}
-
-
-/* structs to store a list of BDR node records */
-typedef struct BdrNodeInfoListCell
-{
-	struct BdrNodeInfoListCell *next;
-	t_bdr_node_info *node_info;
-} BdrNodeInfoListCell;
-
-typedef struct BdrNodeInfoList
-{
-	BdrNodeInfoListCell *head;
-	BdrNodeInfoListCell *tail;
-	int			node_count;
-} BdrNodeInfoList;
-
-#define T_BDR_NODE_INFO_LIST_INITIALIZER { \
-	NULL, \
-	NULL, \
-	0 \
-}
-
-
-
 typedef struct
 {
 	char		filepath[MAXPGPATH];
@@ -372,6 +330,7 @@ typedef struct
 
 #define T_CONFIGFILE_INFO_INITIALIZER { "", "", false }
 
+
 typedef struct
 {
 	int			size;
@@ -381,6 +340,7 @@ typedef struct
 
 #define T_CONFIGFILE_LIST_INITIALIZER { 0, 0, NULL }
 
+
 typedef struct
 {
 	uint64		system_identifier;
@@ -420,10 +380,6 @@ typedef struct RepmgrdInfo {
 /* utility functions */
 
 XLogRecPtr	parse_lsn(const char *str);
-
-extern void
-wrap_ddl_query(PQExpBufferData *query_buf, int replication_type, const char *fmt,...)
-__attribute__((format(PG_PRINTF_ATTRIBUTE, 3, 4)));
 bool		atobool(const char *value);
 
 /* connection functions */
@@ -432,13 +388,19 @@ PGconn	   *establish_db_connection(const char *conninfo,
 PGconn	   *establish_db_connection_quiet(const char *conninfo);
 PGconn	   *establish_db_connection_by_params(t_conninfo_param_list *param_list,
 								  const bool exit_on_error);
+PGconn	   *establish_db_connection_with_replacement_param(const char *conninfo,
+														   const char *param,
+														   const char *value,
+														   const bool exit_on_error);
+PGconn	   *establish_replication_connection_from_conn(PGconn *conn, const char *repluser);
+PGconn	   *establish_replication_connection_from_conninfo(const char *conninfo, const char *repluser);
+
 PGconn	   *establish_primary_db_connection(PGconn *conn,
 								const bool exit_on_error);
 PGconn	   *get_primary_connection(PGconn *standby_conn, int *primary_id, char *primary_conninfo_out);
 PGconn	   *get_primary_connection_quiet(PGconn *standby_conn, int *primary_id, char *primary_conninfo_out);
+PGconn	   *duplicate_connection(PGconn *conn, const char *user, bool replication);
 
-bool		is_superuser_connection(PGconn *conn, t_connection_user *userinfo);
-bool		connection_has_pg_settings(PGconn *conn);
 void		close_connection(PGconn **conn);
 
 /* conninfo manipulation functions */
@@ -467,6 +429,7 @@ bool		set_config(PGconn *conn, const char *config_param, const char *config_valu
 bool		set_config_bool(PGconn *conn, const char *config_param, bool state);
 int		    guc_set(PGconn *conn, const char *parameter, const char *op, const char *value);
 bool		get_pg_setting(PGconn *conn, const char *setting, char *output);
+bool		get_pg_setting_bool(PGconn *conn, const char *setting, bool *output);
 bool		get_pg_setting_int(PGconn *conn, const char *setting, int *output);
 bool		alter_system_int(PGconn *conn, const char *name, int value);
 bool		pg_reload_conf(PGconn *conn);
@@ -482,6 +445,12 @@ bool		identify_system(PGconn *repl_conn, t_system_identification *identification
 uint64		system_identifier(PGconn *conn);
 TimeLineHistoryEntry *get_timeline_history(PGconn *repl_conn, TimeLineID tli);
 
+/* user/role information functions */
+bool		can_execute_pg_promote(PGconn *conn);
+bool		connection_has_pg_settings(PGconn *conn);
+bool		is_replication_role(PGconn *conn, char *rolname);
+bool		is_superuser_connection(PGconn *conn, t_connection_user *userinfo);
+
 /* repmgrd shared memory functions */
 bool		repmgrd_set_local_node_id(PGconn *conn, int local_node_id);
 int			repmgrd_get_local_node_id(PGconn *conn);
@@ -559,10 +528,14 @@ PGresult   *get_event_records(PGconn *conn, int node_id, const char *node_name,
 
 /* replication slot functions */
 void		create_slot_name(char *slot_name, int node_id);
-bool		create_replication_slot(PGconn *conn, char *slot_name, PQExpBufferData *error_msg);
-bool		drop_replication_slot(PGconn *conn, char *slot_name);
+
+bool		create_replication_slot_sql(PGconn *conn, char *slot_name, PQExpBufferData *error_msg);
+bool		create_replication_slot_replprot(PGconn *conn, PGconn *repl_conn, char *slot_name, PQExpBufferData *error_msg);
+bool		drop_replication_slot_sql(PGconn *conn, char *slot_name);
+bool		drop_replication_slot_replprot(PGconn *repl_conn, char *slot_name);
+
 RecordStatus get_slot_record(PGconn *conn, char *slot_name, t_replication_slot *record);
-int			get_free_replication_slot_count(PGconn *conn);
+int			get_free_replication_slot_count(PGconn *conn, int *max_replication_slots);
 int			get_inactive_replication_slots(PGconn *conn, KeyValueList *list);
 
 /* tablespace functions */
@@ -614,7 +587,7 @@ XLogRecPtr	get_last_wal_receive_location(PGconn *conn);
 void		init_replication_info(ReplInfo *replication_info);
 bool		get_replication_info(PGconn *conn, t_server_type node_type, ReplInfo *replication_info);
 int			get_replication_lag_seconds(PGconn *conn);
-TimeLineID	get_node_timeline(PGconn *conn);
+TimeLineID	get_node_timeline(PGconn *conn, char *timeline_id_str);
 void		get_node_replication_stats(PGconn *conn, t_node_info *node_info);
 NodeAttached is_downstream_node_attached(PGconn *conn, char *node_name);
 void		set_upstream_last_seen(PGconn *conn, int upstream_node_id);
@@ -622,27 +595,6 @@ int			get_upstream_last_seen(PGconn *conn, t_server_type node_type);
 
 bool		is_wal_replay_paused(PGconn *conn, bool check_pending_wal);
 
-/* BDR functions */
-int			get_bdr_version_num(void);
-void		get_all_bdr_node_records(PGconn *conn, BdrNodeInfoList *node_list);
-RecordStatus get_bdr_node_record_by_name(PGconn *conn, const char *node_name, t_bdr_node_info *node_info);
-bool		is_bdr_db(PGconn *conn, PQExpBufferData *output);
-bool		is_bdr_db_quiet(PGconn *conn);
-bool		is_active_bdr_node(PGconn *conn, const char *node_name);
-bool		is_bdr_repmgr(PGconn *conn);
-char	   *get_default_bdr_replication_set(PGconn *conn);
-bool		is_table_in_bdr_replication_set(PGconn *conn, const char *tablename, const char *set);
-bool		add_table_to_bdr_replication_set(PGconn *conn, const char *tablename, const char *set);
-void		add_extension_tables_to_bdr_replication_set(PGconn *conn);
-bool		bdr_node_name_matches(PGconn *conn, const char *node_name, PQExpBufferData *bdr_local_node_name);
-ReplSlotStatus get_bdr_node_replication_slot_status(PGconn *conn, const char *node_name);
-void		get_bdr_other_node_name(PGconn *conn, int node_id, char *name_buf);
-
-bool		am_bdr_failover_handler(PGconn *conn, int node_id);
-void		unset_bdr_failover_handler(PGconn *conn);
-bool		bdr_node_has_repmgr_set(PGconn *conn, const char *node_name);
-bool		bdr_node_set_repmgr_set(PGconn *conn, const char *node_name);
-
 /* miscellaneous debugging functions */
 const char *print_node_status(NodeStatus node_status);
 const char *print_pqping_status(PGPing ping_status);
