@@ -3,7 +3,7 @@
  *
  * Implements cluster information actions for the repmgr command line utility
  *
- * Copyright (c) 2ndQuadrant, 2010-2019
+ * Copyright (c) 2ndQuadrant, 2010-2020
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,6 +67,8 @@ static void cube_set_node_status(t_node_status_cube **cube, int n, int node_id,
  * Parameters:
  *   --compact
  *   --csv
+ *   --terse
+ *   --verbose
  */
 void
 do_cluster_show(void)
@@ -206,7 +208,8 @@ do_cluster_show(void)
 		else
 		{
 			/* NOP on pre-9.6 servers */
-			cell->node_info->replication_info->timeline_id = get_node_timeline(cell->node_info->conn);
+			cell->node_info->replication_info->timeline_id = get_node_timeline(cell->node_info->conn,
+																			   cell->node_info->replication_info->timeline_id_str);
 		}
 
 		initPQExpBuffer(&node_status);
@@ -244,18 +247,13 @@ do_cluster_show(void)
 
 		headers_show[SHOW_LOCATION].cur_length = strlen(cell->node_info->location);
 
-		if (cell->node_info->replication_info->timeline_id == UNKNOWN_TIMELINE_ID)
+		/* Format timeline ID */
+		if (cell->node_info->type == WITNESS)
 		{
-			/* display "?" */
-			headers_show[SHOW_PRIORITY].cur_length = 1;
-		}
-		else
-		{
-			initPQExpBuffer(&buf);
-			appendPQExpBuffer(&buf, "%i", cell->node_info->replication_info->timeline_id);
-			headers_show[SHOW_PRIORITY].cur_length = strlen(buf.data);
-			termPQExpBuffer(&buf);
+			/* The witness node's timeline ID is irrelevant */
+			strncpy(cell->node_info->replication_info->timeline_id_str, _("n/a"), MAXLEN);
 		}
+		headers_show[SHOW_TIMELINE_ID].cur_length = strlen(cell->node_info->replication_info->timeline_id_str);
 
 		headers_show[SHOW_CONNINFO].cur_length = strlen(cell->node_info->conninfo);
 
@@ -322,10 +320,7 @@ do_cluster_show(void)
 
 			if (headers_show[SHOW_TIMELINE_ID].display == true)
 			{
-				if (cell->node_info->replication_info->timeline_id == UNKNOWN_TIMELINE_ID)
-					printf("| %-*c ", headers_show[SHOW_TIMELINE_ID].max_length, '?');
-				else
-					printf("| %-*i ", headers_show[SHOW_TIMELINE_ID].max_length, (int)cell->node_info->replication_info->timeline_id);
+				printf("| %-*s ", headers_show[SHOW_TIMELINE_ID].max_length, cell->node_info->replication_info->timeline_id_str);
 			}
 
 			if (headers_show[SHOW_CONNINFO].display == true)
@@ -343,14 +338,25 @@ do_cluster_show(void)
 	/* emit any warnings */
 	if (warnings.head != NULL && runtime_options.terse == false && runtime_options.output_mode != OM_CSV)
 	{
-		ItemListCell *cell = NULL;
 
-		printf(_("\nWARNING: following issues were detected\n"));
+		ItemListCell *cell = NULL;
+		PQExpBufferData warning;
+
+		initPQExpBuffer(&warning);
+
+		appendPQExpBufferStr(&warning,
+							 _("following issues were detected\n"));
 		for (cell = warnings.head; cell; cell = cell->next)
 		{
-			printf(_("  - %s\n"), cell->string);
+			appendPQExpBuffer(&warning,
+							  _("  - %s\n"), cell->string);
 		}
 
+		puts("");
+		log_warning("%s", warning.data);
+
+		termPQExpBuffer(&warning);
+
 		if (runtime_options.verbose == false && connection_error_found == true)
 		{
 			log_hint(_("execute with --verbose option to see connection error messages"));
@@ -1396,6 +1402,10 @@ do_cluster_cleanup(void)
 		log_warning(_("unable to vacuum table \"repmgr.monitoring_history\""));
 		log_detail("%s", PQerrorMessage(primary_conn));
 	}
+	else
+	{
+		log_info(_("vacuum of table \"repmgr.monitoring_history\" completed"));
+	}
 
 	if (runtime_options.keep_history == 0)
 	{
