@@ -3,7 +3,7 @@
  *
  * Implements actions available for any kind of node
  *
- * Copyright (c) 2ndQuadrant, 2010-2019
+ * Copyright (c) 2ndQuadrant, 2010-2020
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,12 +44,13 @@ static void _do_node_restore_config(void);
 static void do_node_check_replication_connection(void);
 static CheckStatus do_node_check_archive_ready(PGconn *conn, OutputMode mode, CheckStatusList *list_output);
 static CheckStatus do_node_check_downstream(PGconn *conn, OutputMode mode, CheckStatusList *list_output);
+static CheckStatus do_node_check_upstream(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output);
 static CheckStatus do_node_check_replication_lag(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output);
 static CheckStatus do_node_check_role(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output);
 static CheckStatus do_node_check_slots(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output);
 static CheckStatus do_node_check_missing_slots(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output);
 static CheckStatus do_node_check_data_directory(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output);
-
+static CheckStatus do_node_check_replication_config_owner(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output);
 /*
  * NODE STATUS
  *
@@ -159,7 +160,6 @@ do_node_status(void)
 								 _("- node is registered as standby but running as primary"));
 			}
 			break;
-		case BDR:
 		default:
 			break;
 	}
@@ -717,10 +717,49 @@ do_node_check(void)
 		exit(SUCCESS);
 	}
 
-	if (strlen(config_file_options.conninfo))
-		conn = establish_db_connection(config_file_options.conninfo, true);
+
+	if (config_file_options.conninfo[0] != '\0')
+	{
+		t_conninfo_param_list node_conninfo = T_CONNINFO_PARAM_LIST_INITIALIZER;
+		char	   *errmsg = NULL;
+		bool		parse_success = false;
+
+		initialize_conninfo_params(&node_conninfo, false);
+
+		parse_success = parse_conninfo_string(config_file_options.conninfo,
+											  &node_conninfo,
+											  &errmsg, false);
+
+		if (parse_success == false)
+		{
+			log_error(_("unable to parse conninfo string \"%s\" for local node"),
+					  config_file_options.conninfo);
+			log_detail("%s", errmsg);
+
+			exit(ERR_BAD_CONFIG);
+		}
+
+		/*
+		 * If --superuser option provided, attempt to connect as the specified user
+		 */
+
+		if (runtime_options.superuser[0] != '\0')
+		{
+			conn = establish_db_connection_with_replacement_param(
+				config_file_options.conninfo,
+				"user",
+				runtime_options.superuser,
+				true);
+		}
+		else
+		{
+			conn = establish_db_connection_by_params(&node_conninfo, true);
+		}
+	}
 	else
+	{
 		conn = establish_db_connection_by_params(&source_conninfo, true);
+	}
 
 	if (get_node_record(conn, config_file_options.node_id, &node_info) != RECORD_FOUND)
 	{
@@ -744,6 +783,16 @@ do_node_check(void)
 		exit(return_code);
 	}
 
+	if (runtime_options.upstream == true)
+	{
+		return_code = do_node_check_upstream(conn,
+											 runtime_options.output_mode,
+											 &node_info,
+											 NULL);
+		PQfinish(conn);
+		exit(return_code);
+	}
+
 	if (runtime_options.downstream == true)
 	{
 		return_code = do_node_check_downstream(conn,
@@ -753,7 +802,6 @@ do_node_check(void)
 		exit(return_code);
 	}
 
-
 	if (runtime_options.replication_lag == true)
 	{
 		return_code = do_node_check_replication_lag(conn,
@@ -804,6 +852,16 @@ do_node_check(void)
 		exit(return_code);
 	}
 
+	if (runtime_options.replication_config_owner == true)
+	{
+		return_code = do_node_check_replication_config_owner(conn,
+													   runtime_options.output_mode,
+													   &node_info,
+													   NULL);
+		PQfinish(conn);
+		exit(return_code);
+	}
+
 
 	if (runtime_options.output_mode == OM_NAGIOS)
 	{
@@ -827,6 +885,9 @@ do_node_check(void)
 	if (do_node_check_archive_ready(conn, runtime_options.output_mode, &status_list) != CHECK_STATUS_OK)
 		issue_detected = true;
 
+	if (do_node_check_upstream(conn, runtime_options.output_mode, &node_info, &status_list) != CHECK_STATUS_OK)
+		issue_detected = true;
+
 	if (do_node_check_downstream(conn, runtime_options.output_mode, &status_list) != CHECK_STATUS_OK)
 		issue_detected = true;
 
@@ -909,7 +970,6 @@ do_node_check_replication_connection(void)
 	PGconn *repl_conn = NULL;
 	t_node_info node_record = T_NODE_INFO_INITIALIZER;
 	RecordStatus record_status = RECORD_NOT_FOUND;
-	t_conninfo_param_list remote_conninfo = T_CONNINFO_PARAM_LIST_INITIALIZER;
 	PQExpBufferData output;
 
 
@@ -939,17 +999,8 @@ do_node_check_replication_connection(void)
 		return;
 	}
 
-	initialize_conninfo_params(&remote_conninfo, false);
-	parse_conninfo_string(node_record.conninfo, &remote_conninfo, NULL, false);
-
-	if (strcmp(param_get(&remote_conninfo, "user"), node_record.repluser) != 0)
-	{
-		param_set(&remote_conninfo, "user", node_record.repluser);
-		param_set(&remote_conninfo, "dbname", "replication");
-	}
-	param_set(&remote_conninfo, "replication", "1");
-
-	repl_conn = establish_db_connection_by_params(&remote_conninfo, false);
+	repl_conn = establish_replication_connection_from_conninfo(node_record.conninfo,
+															   node_record.repluser);
 
 	if (PQstatus(repl_conn) != CONNECTION_OK)
 	{
@@ -1298,6 +1349,98 @@ do_node_check_downstream(PGconn *conn, OutputMode mode, CheckStatusList *list_ou
 }
 
 
+static CheckStatus
+do_node_check_upstream(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output)
+{
+	PGconn	   *upstream_conn = NULL;
+	t_node_info upstream_node_info = T_NODE_INFO_INITIALIZER;
+	PQExpBufferData details;
+
+	CheckStatus status = CHECK_STATUS_OK;
+
+	if (mode == OM_CSV && list_output == NULL)
+	{
+		log_error(_("--csv output not provided with --upstream option"));
+		PQfinish(conn);
+		exit(ERR_BAD_CONFIG);
+	}
+
+	initPQExpBuffer(&details);
+
+	if (get_node_record(conn, node_info->upstream_node_id, &upstream_node_info) != RECORD_FOUND)
+	{
+		if (get_recovery_type(conn) == RECTYPE_STANDBY)
+		{
+			appendPQExpBuffer(&details,
+							  _("node \"%s\" (ID: %i) is a standby but no upstream record found"),
+							  node_info->node_name,
+							  node_info->node_id);
+			status = CHECK_STATUS_CRITICAL;
+		}
+		else
+		{
+			appendPQExpBufferStr(&details,
+								 _("N/A - node is primary"));
+		}
+	}
+	else
+	{
+		upstream_conn = establish_db_connection(upstream_node_info.conninfo, true);
+
+		/* check our node is connected */
+		if (is_downstream_node_attached(upstream_conn, config_file_options.node_name) != NODE_ATTACHED)
+		{
+			appendPQExpBuffer(&details,
+							  _("node \"%s\" (ID: %i) is not attached to expected upstream node \"%s\" (ID: %i)"),
+							  node_info->node_name,
+							  node_info->node_id,
+							  upstream_node_info.node_name,
+							  upstream_node_info.node_id);
+			status = CHECK_STATUS_CRITICAL;
+		}
+		else
+		{
+			appendPQExpBuffer(&details,
+							  _("node \"%s\" (ID: %i) is attached to expected upstream node \"%s\" (ID: %i)"),
+							  node_info->node_name,
+							  node_info->node_id,
+							  upstream_node_info.node_name,
+							  upstream_node_info.node_id);
+		}
+	}
+
+	switch (mode)
+	{
+		case OM_NAGIOS:
+			{
+				printf("REPMGR_UPSTREAM_SERVER %s: %s | ",
+					   output_check_status(status),
+					   details.data);
+			}
+		case OM_TEXT:
+			if (list_output != NULL)
+			{
+				check_status_list_set(list_output,
+									  "Upstream connection",
+									  status,
+									  details.data);
+			}
+			else
+			{
+				printf("%s (%s)\n",
+					   output_check_status(status),
+					   details.data);
+			}
+		default:
+			break;
+	}
+
+	termPQExpBuffer(&details);
+
+	return status;
+}
+
+
 static CheckStatus
 do_node_check_replication_lag(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output)
 {
@@ -1552,34 +1695,6 @@ do_node_check_role(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckS
 									 _("node is witness"));
 			}
 			break;
-		case BDR:
-			{
-				PQExpBufferData output;
-
-				initPQExpBuffer(&output);
-				if (is_bdr_db(conn, &output) == false)
-				{
-					status = CHECK_STATUS_CRITICAL;
-					appendPQExpBufferStr(&details,
-										 output.data);
-				}
-				termPQExpBuffer(&output);
-
-				if (status == CHECK_STATUS_OK)
-				{
-					if (is_active_bdr_node(conn, node_info->node_name) == false)
-					{
-						status = CHECK_STATUS_CRITICAL;
-						appendPQExpBufferStr(&details,
-											 _("node is not an active BDR node"));
-					}
-					else
-					{
-						appendPQExpBufferStr(&details,
-											 _("node is an active BDR node"));
-					}
-				}
-			}
 		default:
 			break;
 	}
@@ -1828,11 +1943,11 @@ do_node_check_data_directory(PGconn *conn, OutputMode mode, t_node_info *node_in
 	}
 
 	initPQExpBuffer(&details);
+
 	/*
 	 * Check actual data directory matches that in repmgr.conf; note this requires
 	 * a superuser connection
 	 */
-
 	if (connection_has_pg_settings(conn) == true)
 	{
 		/* we expect to have a database connection */
@@ -1875,10 +1990,9 @@ do_node_check_data_directory(PGconn *conn, OutputMode mode, t_node_info *node_in
 		{
 			log_info(_("connection is not a superuser connection, falling back to simple check"));
 
-			/* XXX add -S/--superuser option */
 			if (PQserverVersion(conn) >= 100000)
 			{
-				log_hint(_("add the \"%s\" user to group \"pg_read_all_settings\" or \"pg_monitor\""),
+				log_hint(_("provide a superuser with -S/--superuser, or add the \"%s\" user to role \"pg_read_all_settings\" or \"pg_monitor\""),
 						   PQuser(conn));
 			}
 		}
@@ -1899,6 +2013,12 @@ do_node_check_data_directory(PGconn *conn, OutputMode mode, t_node_info *node_in
 
 			status = CHECK_STATUS_CRITICAL;
 		}
+		else
+		{
+			appendPQExpBuffer(&details,
+							  _("configured \"data_directory\" is \"%s\""),
+							  config_file_options.data_directory);
+		}
 	}
 
 	switch (mode)
@@ -1942,6 +2062,40 @@ do_node_check_data_directory(PGconn *conn, OutputMode mode, t_node_info *node_in
 	return status;
 }
 
+/*
+ * This is not included in the general list output
+ */
+static
+CheckStatus do_node_check_replication_config_owner(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output)
+{
+	CheckStatus status = CHECK_STATUS_OK;
+
+	PQExpBufferData errmsg;
+	PQExpBufferData details;
+
+	if (mode != OM_OPTFORMAT)
+	{
+		log_error(_("--replication-config-owner option can only be used with --optformat"));
+		PQfinish(conn);
+		exit(ERR_BAD_CONFIG);
+	}
+
+	initPQExpBuffer(&errmsg);
+	initPQExpBuffer(&details);
+
+	if (check_replication_config_owner(PQserverVersion(conn),
+									   config_file_options.data_directory,
+									   &errmsg, &details) == false)
+	{
+		status = CHECK_STATUS_CRITICAL;
+	}
+
+	printf("--replication-config-owner=%s\n",
+		   output_check_status(status));
+
+	return status;
+}
+
 
 void
 do_node_service(void)
@@ -1981,26 +2135,62 @@ do_node_service(void)
 
 	if ((action == ACTION_STOP || action == ACTION_RESTART) && runtime_options.checkpoint == true)
 	{
-		if (runtime_options.dry_run == true)
+		PGconn	   *conn = NULL;
+
+		if (config_file_options.conninfo[0] != '\0')
 		{
-			log_info(_("a CHECKPOINT would be issued here"));
+			/*
+			 * If --superuser option provided, attempt to connect as the specified user
+			 */
+			if (runtime_options.superuser[0] != '\0')
+			{
+				conn = establish_db_connection_with_replacement_param(
+					config_file_options.conninfo,
+					"user",
+					runtime_options.superuser,
+					true);
+			}
+			else
+			{
+				conn = establish_db_connection(config_file_options.conninfo, true);
+			}
 		}
 		else
 		{
-			PGconn	   *conn = NULL;
-
-			if (strlen(config_file_options.conninfo))
-				conn = establish_db_connection(config_file_options.conninfo, true);
-			else
-				conn = establish_db_connection_by_params(&source_conninfo, true);
-
-			log_notice(_("issuing CHECKPOINT"));
-
-			/* check superuser conn! */
-			checkpoint(conn);
-
-			PQfinish(conn);
+			conn = establish_db_connection_by_params(&source_conninfo, true);
 		}
+
+		if (is_superuser_connection(conn, NULL) == false)
+		{
+			if (runtime_options.dry_run == true)
+			{
+				log_warning(_("a CHECKPOINT would be issued here but no superuser connection is available"));
+			}
+			else
+			{
+				log_warning(_("a superuser connection is required to issue a CHECKPOINT"));
+			}
+
+			log_hint(_("provide a superuser with -S/--superuser"));
+		}
+		else
+		{
+			if (runtime_options.dry_run == true)
+			{
+				log_info(_("a CHECKPOINT would be issued here"));
+			}
+			else
+			{
+
+				log_notice(_("issuing CHECKPOINT on node \"%s\" (ID: %i) "),
+						   config_file_options.node_name,
+						   config_file_options.node_id);
+
+				checkpoint(conn);
+			}
+		}
+
+		PQfinish(conn);
 	}
 
 	get_server_action(action, command, data_dir);
@@ -2147,6 +2337,7 @@ do_node_rejoin(void)
 	PQExpBufferData follow_output;
 	struct stat statbuf;
 	t_node_info primary_node_record = T_NODE_INFO_INITIALIZER;
+	t_node_info local_node_record = T_NODE_INFO_INITIALIZER;
 
 	bool		success = true;
 	int			follow_error_code = SUCCESS;
@@ -2257,6 +2448,36 @@ do_node_rejoin(void)
 		exit(ERR_BAD_CONFIG);
 	}
 
+	/*
+	 * Fetch the local node record - we'll need this later, and it acts as an
+	 * additional sanity-check that the node is known to the primary.
+	 */
+	if (get_node_record(primary_conn, config_file_options.node_id, &local_node_record) != RECORD_FOUND)
+	{
+		log_error(_("unable to retrieve node record for the local node"));
+		log_hint(_("check the local node is registered with the current primary \"%s\" (ID: %i)"),
+				 primary_node_record.node_name,
+				 primary_node_record.node_id);
+		PQfinish(upstream_conn);
+		PQfinish(primary_conn);
+		exit(ERR_BAD_CONFIG);
+	}
+
+	/*
+	 * Sanity-check replication slot availability
+	 */
+	if (config_file_options.use_replication_slots)
+	{
+		bool slots_available = check_replication_slots_available(primary_node_record.node_id,
+																 primary_conn);
+		if (slots_available == false)
+		{
+			PQfinish(primary_conn);
+			exit(ERR_BAD_CONFIG);
+		}
+	}
+
+
 	/*
 	 * sanity-check that it will actually be possible to stream from the new upstream
 	 */
@@ -2534,79 +2755,34 @@ do_node_rejoin(void)
 	 */
 	if (runtime_options.no_wait == false)
 	{
-		int i;
-
-		for (i = 0; i < config_file_options.standby_reconnect_timeout; i++)
-		{
-			if (is_server_available(config_file_options.conninfo))
-			{
-				log_verbose(LOG_INFO, _("demoted primary is pingable"));
-				break;
-			}
-
-			if (i % 5 == 0)
-			{
-				log_verbose(LOG_INFO, _("waiting for node %i to respond to pings; %i of max %i attempts"),
-							config_file_options.node_id,
-							i + 1, config_file_options.node_rejoin_timeout);
-			}
-			else
-			{
-				log_debug("sleeping 1 second waiting for node %i to respond to pings; %i of max %i attempts",
-						  config_file_options.node_id,
-						  i + 1, config_file_options.node_rejoin_timeout);
-			}
-
-			sleep(1);
-		}
-
-		for (;  i < config_file_options.node_rejoin_timeout; i++)
-		{
-			NodeAttached node_attached = is_downstream_node_attached(primary_conn,
-														config_file_options.node_name);
-
-			if (node_attached == NODE_ATTACHED)
-			{
-				log_verbose(LOG_INFO, _("node %i has attached to its upstream node"),
-							config_file_options.node_id);
-				break;
-			}
-
-			if (i % 5 == 0)
-			{
-				log_info(_("waiting for node \"%s\" (ID: %i) to connect to new primary; %i of max %i attempts"),
-						 config_file_options.node_name,
-						 config_file_options.node_id,
-						 i + 1, config_file_options.node_rejoin_timeout);
-				log_detail(_("checking for record in node \"%s\"'s \"pg_stat_replication\" table where \"application_name\" is \"%s\""),
-						   primary_node_record.node_name,
-						   config_file_options.node_name);
-			}
-			else
-			{
-				log_debug("sleeping 1 second waiting for node %i to connect to new primary; %i of max %i attempts",
-						  config_file_options.node_id,
-						  i + 1, config_file_options.node_rejoin_timeout);
-			}
-
-			sleep(1);
-		}
+		standy_join_status join_success = check_standby_join(primary_conn,
+															 &primary_node_record,
+															 &local_node_record);
 
 		create_event_notification(primary_conn,
 								  &config_file_options,
 								  config_file_options.node_id,
 								  "node_rejoin",
-								  success,
+								  join_success == JOIN_SUCCESS ? true : false,
 								  follow_output.data);
 
-		if (success == false)
+		if (join_success != JOIN_SUCCESS)
 		{
 			termPQExpBuffer(&follow_output);
 			log_error(_("NODE REJOIN failed"));
-			log_detail(_("no record for local node \"%s\" found in node \"%s\"'s \"pg_stat_replication\" table"),
-					   config_file_options.node_name,
-					   primary_node_record.node_name);
+
+			if (join_success == JOIN_FAIL_NO_PING) {
+				log_detail(_("local node \"%s\" did not become available start after %i seconds"),
+						   config_file_options.node_name,
+						   config_file_options.node_rejoin_timeout);
+			}
+			else {
+				log_detail(_("no record for local node \"%s\" found in node \"%s\"'s \"pg_stat_replication\" table"),
+						   config_file_options.node_name,
+						   primary_node_record.node_name);
+			}
 			log_hint(_("check the PostgreSQL log on the local node"));
+
 			exit(ERR_REJOIN_FAIL);
 		}
 	}
@@ -3173,17 +3349,22 @@ do_node_help(void)
 	puts("");
 	printf(_("  Configuration file required, runs on local node only.\n"));
 	puts("");
-	printf(_("    --csv                   emit output as CSV (not available for individual check output)\n"));
-	printf(_("    --nagios                emit output in Nagios format (individual check output only)\n"));
+	printf(_("  Connection options:\n"));
+	printf(_("    -S, --superuser=USERNAME  superuser to use, if repmgr user is not superuser\n"));
+	puts("");
+	printf(_("  Output options:\n"));
+	printf(_("    --csv                     emit output as CSV (not available for individual check output)\n"));
+	printf(_("    --nagios                  emit output in Nagios format (individual check output only)\n"));
 	puts("");
 	printf(_("  Following options check an individual status:\n"));
-	printf(_("    --archive-ready         number of WAL files ready for archiving\n"));
-	printf(_("    --downstream            whether all downstream nodes are connected\n"));
-	printf(_("    --replication-lag       replication lag in seconds (standbys only)\n"));
-	printf(_("    --role                  check node has expected role\n"));
-	printf(_("    --slots                 check for inactive replication slots\n"));
-	printf(_("    --missing-slots         check for missing replication slots\n"));
-	printf(_("    --data-directory-config check repmgr's data directory configuration\n"));
+	printf(_("    --archive-ready           number of WAL files ready for archiving\n"));
+	printf(_("    --downstream              whether all downstream nodes are connected\n"));
+	printf(_("    --uptream                 whether the node is connected to its upstream\n"));
+	printf(_("    --replication-lag         replication lag in seconds (standbys only)\n"));
+	printf(_("    --role                    check node has expected role\n"));
+	printf(_("    --slots                   check for inactive replication slots\n"));
+	printf(_("    --missing-slots           check for missing replication slots\n"));
+	printf(_("    --data-directory-config   check repmgr's data directory configuration\n"));
 
 	puts("");
 
@@ -3202,7 +3383,7 @@ do_node_help(void)
 			 "                            after executing \"pg_rewind\"\n"));
 	printf(_("    --config-archive-dir    directory to temporarily store retained configuration files\n" \
 			 "                              (default: /tmp)\n"));
-	printf(_("    -W/--no-wait            don't wait for the node to rejoin cluster\n"));
+	printf(_("    -W, --no-wait           don't wait for the node to rejoin cluster\n"));
 	puts("");
 
 	printf(_("NODE SERVICE\n"));
@@ -3212,10 +3393,11 @@ do_node_help(void)
 	puts("");
 	printf(_("  Configuration file required, runs on local node only.\n"));
 	puts("");
-	printf(_("    --dry-run             show what action would be performed, but don't execute it\n"));
-	printf(_("    --action              action to perform (one of \"start\", \"stop\", \"restart\" or \"reload\")\n"));
-	printf(_("    --list-actions        show what command would be performed for each action\n"));
-	printf(_("    --checkpoint          issue a CHECKPOINT before stopping or restarting the node\n"));
+	printf(_("    --dry-run                 show what action would be performed, but don't execute it\n"));
+	printf(_("    --action                  action to perform (one of \"start\", \"stop\", \"restart\" or \"reload\")\n"));
+	printf(_("    --list-actions            show what command would be performed for each action\n"));
+	printf(_("    --checkpoint              issue a CHECKPOINT before stopping or restarting the node\n"));
+	printf(_("    -S, --superuser=USERNAME  superuser to use, if repmgr user is not superuser\n"));
 	puts("");
 
 
