@@ -1,6 +1,6 @@
 /*
  * repmgr-client-global.h
- * Copyright (c) 2ndQuadrant, 2010-2019
+ * Copyright (c) 2ndQuadrant, 2010-2020
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,8 @@
 /* default value for "cluster event --limit"*/
 #define CLUSTER_EVENT_LIMIT 20
 
+
+
 typedef struct
 {
 	/* configuration metadata */
@@ -85,7 +87,7 @@ typedef struct
 	char		replication_user[MAXLEN];
 	char		upstream_conninfo[MAXLEN];
 	bool		without_barman;
-	bool		recovery_conf_only;
+	bool		replication_conf_only;
 
 	/* "standby clone"/"standby follow" options */
 	int			upstream_node_id;
@@ -109,6 +111,7 @@ typedef struct
 	/* "node check" options */
 	bool		archive_ready;
 	bool		downstream;
+	bool		upstream;
 	bool		replication_lag;
 	bool		role;
 	bool		slots;
@@ -116,6 +119,7 @@ typedef struct
 	bool		has_passfile;
 	bool		replication_connection;
 	bool		data_directory_config;
+	bool		replication_config_owner;
 
 	/* "node rejoin" options */
 	char		config_files[MAXLEN];
@@ -168,7 +172,7 @@ typedef struct
 		/* "node status" options */ \
 		false, \
 		/* "node check" options */ \
-		false, false, false, false, false, false, false, false,	false, \
+		false, false, false, false, false, false, false, false,	false, false, false, \
 		/* "node rejoin" options */ \
 		"", \
 		/* "node service" options */ \
@@ -201,6 +205,23 @@ typedef enum
 } t_server_action;
 
 
+typedef enum
+{
+	USER_TYPE_UNKNOWN = -1,
+	REPMGR_USER,
+	REPLICATION_USER_OPT,
+	REPLICATION_USER_NODE,
+	SUPERUSER
+} t_user_type;
+
+typedef enum
+{
+	JOIN_SUCCESS,
+	JOIN_FAIL_NO_PING,
+	JOIN_FAIL_NO_REPLICATION
+} standy_join_status;
+
+
 typedef struct ColHeader
 {
 	char		title[MAXLEN];
@@ -255,9 +276,15 @@ extern void get_node_config_directory(char *config_dir_buf);
 extern void get_node_data_directory(char *data_dir_buf);
 extern void init_node_record(t_node_info *node_record);
 extern bool can_use_pg_rewind(PGconn *conn, const char *data_directory, PQExpBufferData *reason);
-extern void drop_replication_slot_if_exists(PGconn *conn, int node_id, char *slot_name);
 
+extern bool create_replication_slot(PGconn *conn, char *slot_name, t_node_info *upstream_node_record, PQExpBufferData *error_msg);
+extern bool drop_replication_slot_if_exists(PGconn *conn, int node_id, char *slot_name);
+
+extern standy_join_status check_standby_join(PGconn *primary_conn, t_node_info *primary_node_record, t_node_info *standby_node_record);
+extern bool check_replication_slots_available(int node_id, PGconn* conn);
 extern bool check_node_can_attach(TimeLineID local_tli, XLogRecPtr local_xlogpos, PGconn *follow_target_conn, t_node_info *follow_target_node_record, bool is_rejoin);
+extern bool check_replication_config_owner(int pg_version, const char *data_directory, PQExpBufferData *error_msg, PQExpBufferData *detail_msg);
+
 extern void check_shared_library(PGconn *conn);
 extern bool is_repmgrd_running(PGconn *conn);
 extern int parse_repmgr_version(const char *version_string);
