@@ -1,6 +1,6 @@
 /*
  * repmgr-client.h
- * Copyright (c) 2ndQuadrant, 2010-2019
+ * Copyright (c) 2ndQuadrant, 2010-2020
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,23 +34,21 @@
 #define STANDBY_SWITCHOVER	   8
 #define WITNESS_REGISTER       9
 #define WITNESS_UNREGISTER     10
-#define BDR_REGISTER		   11
-#define BDR_UNREGISTER		   12
-#define NODE_STATUS			   13
-#define NODE_CHECK			   14
-#define NODE_SERVICE		   15
-#define NODE_REJOIN            16
-#define NODE_CONTROL           17
-#define CLUSTER_SHOW		   18
-#define CLUSTER_CLEANUP		   19
-#define CLUSTER_MATRIX		   20
-#define CLUSTER_CROSSCHECK	   21
-#define CLUSTER_EVENT		   22
-#define SERVICE_STATUS		   23
-#define SERVICE_PAUSE		   24
-#define SERVICE_UNPAUSE		   25
-#define DAEMON_START 		   26
-#define DAEMON_STOP 		   27
+#define NODE_STATUS			   11
+#define NODE_CHECK			   12
+#define NODE_SERVICE		   13
+#define NODE_REJOIN            14
+#define NODE_CONTROL           15
+#define CLUSTER_SHOW		   16
+#define CLUSTER_CLEANUP		   17
+#define CLUSTER_MATRIX		   18
+#define CLUSTER_CROSSCHECK	   19
+#define CLUSTER_EVENT		   20
+#define SERVICE_STATUS		   21
+#define SERVICE_PAUSE		   22
+#define SERVICE_UNPAUSE		   23
+#define DAEMON_START 		   24
+#define DAEMON_STOP 		   25
 
 /* command line options without short versions */
 #define OPT_HELP						   1001
@@ -83,23 +81,25 @@
 #define OPT_SIBLINGS_FOLLOW				   1028
 #define OPT_ROLE						   1029
 #define OPT_DOWNSTREAM					   1030
-#define OPT_SLOTS						   1031
-#define OPT_CONFIG_ARCHIVE_DIR			   1032
-#define OPT_HAS_PASSFILE				   1033
-#define OPT_WAIT_START					   1034
-#define OPT_REPL_CONN					   1035
-#define OPT_REMOTE_NODE_ID				   1036
-#define OPT_RECOVERY_CONF_ONLY			   1037
-#define OPT_NO_WAIT						   1038
-#define OPT_MISSING_SLOTS				   1039
-#define OPT_REPMGRD_NO_PAUSE			   1040
-#define OPT_VERSION_NUMBER				   1041
-#define OPT_DATA_DIRECTORY_CONFIG		   1042
-#define OPT_COMPACT						   1043
-#define OPT_DISABLE_WAL_RECEIVER		   1044
-#define OPT_ENABLE_WAL_RECEIVER			   1045
-#define OPT_DETAIL						   1046
-#define OPT_REPMGRD_FORCE_UNPAUSE		   1047
+#define OPT_UPSTREAM					   1031
+#define OPT_SLOTS						   1032
+#define OPT_CONFIG_ARCHIVE_DIR			   1033
+#define OPT_HAS_PASSFILE				   1034
+#define OPT_WAIT_START					   1035
+#define OPT_REPL_CONN					   1036
+#define OPT_REMOTE_NODE_ID				   1037
+#define OPT_REPLICATION_CONF_ONLY		   1038
+#define OPT_NO_WAIT						   1039
+#define OPT_MISSING_SLOTS				   1040
+#define OPT_REPMGRD_NO_PAUSE			   1041
+#define OPT_VERSION_NUMBER				   1042
+#define OPT_DATA_DIRECTORY_CONFIG		   1043
+#define OPT_COMPACT						   1044
+#define OPT_DISABLE_WAL_RECEIVER		   1045
+#define OPT_ENABLE_WAL_RECEIVER			   1046
+#define OPT_DETAIL						   1047
+#define OPT_REPMGRD_FORCE_UNPAUSE		   1048
+#define OPT_REPLICATION_CONFIG_OWNER	   1049
 
 /* deprecated since 4.0 */
 #define OPT_CHECK_UPSTREAM_CONFIG		    999
@@ -157,7 +157,9 @@ static struct option long_options[] =
 	{"upstream-conninfo", required_argument, NULL, OPT_UPSTREAM_CONNINFO},
 	{"upstream-node-id", required_argument, NULL, OPT_UPSTREAM_NODE_ID},
 	{"without-barman", no_argument, NULL, OPT_WITHOUT_BARMAN},
-	{"recovery-conf-only", no_argument, NULL, OPT_RECOVERY_CONF_ONLY},
+	{"replication-conf-only", no_argument, NULL, OPT_REPLICATION_CONF_ONLY},
+	/* deprecate this once Pg11 and earlier are unsupported */
+	{"recovery-conf-only", no_argument, NULL, OPT_REPLICATION_CONF_ONLY},
 
 /* "standby register" options */
 	{"wait-start", required_argument, NULL, OPT_WAIT_START},
@@ -178,6 +180,7 @@ static struct option long_options[] =
 /* "node check" options */
 	{"archive-ready", no_argument, NULL, OPT_ARCHIVE_READY},
 	{"downstream", no_argument, NULL, OPT_DOWNSTREAM},
+	{"upstream", no_argument, NULL, OPT_UPSTREAM},
 	{"replication-lag", no_argument, NULL, OPT_REPLICATION_LAG},
 	{"role", no_argument, NULL, OPT_ROLE},
 	{"slots", no_argument, NULL, OPT_SLOTS},
@@ -185,6 +188,7 @@ static struct option long_options[] =
 	{"has-passfile", no_argument, NULL, OPT_HAS_PASSFILE},
 	{"replication-connection", no_argument, NULL, OPT_REPL_CONN},
 	{"data-directory-config", no_argument, NULL, OPT_DATA_DIRECTORY_CONFIG},
+	{"replication-config-owner", no_argument, NULL, OPT_REPLICATION_CONFIG_OWNER},
 
 /* "node rejoin" options */
 	{"config-files", required_argument, NULL, OPT_CONFIG_FILES},
