@@ -1,6 +1,6 @@
 /*
  * repmgr.h
- * Copyright (c) 2ndQuadrant, 2010-2019
+ * Copyright (c) 2ndQuadrant, 2010-2020
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -78,16 +78,15 @@
 #define MIN_SUPPORTED_VERSION_NUM	90300
 
 #define REPLICATION_TYPE_PHYSICAL 1
-#define REPLICATION_TYPE_BDR	  2
 
 #define UNKNOWN_SERVER_VERSION_NUM -1
-#define UNKNOWN_BDR_VERSION_NUM -1
 #define UNKNOWN_REPMGR_VERSION_NUM -1
 
 #define UNKNOWN_TIMELINE_ID -1
 #define UNKNOWN_SYSTEM_IDENTIFIER 0
 #define UNKNOWN_PID			-1
 #define UNKNOWN_REPLICATION_LAG	-1
+#define UNKNOWN_VALUE		-1
 
 #define NODE_NOT_FOUND		-1
 #define NO_UPSTREAM_NODE	-1
@@ -98,8 +97,6 @@
 #define ARCHIVE_STATUS_DIR_ERROR -1
 #define NO_DEGRADED_MONITORING_ELAPSED -1
 
-#define BDR2_REPLICATION_SET_NAME "repmgr"
-
 /*
  * various default values - ensure repmgr.conf.sample is update
  * if any of these are changed
@@ -113,7 +110,6 @@
 #define DEFAULT_PRIMARY_NOTIFICATION_TIMEOUT 60  /* seconds */
 #define DEFAULT_PRIMARY_FOLLOW_TIMEOUT       60  /* seconds */
 #define DEFAULT_STANDBY_FOLLOW_TIMEOUT       30  /* seconds */
-#define DEFAULT_BDR_RECOVERY_TIMEOUT         30  /* seconds */
 #define DEFAULT_ARCHIVE_READY_WARNING        16  /* WAL files */
 #define DEFAULT_ARCHIVE_READY_CRITICAL       128 /* WAL files */
 #define	DEFAULT_REPLICATION_LAG_WARNING      300 /* seconds */
